@@ -567,7 +567,7 @@ func (a *App) ListReleases(c ListConfigProvider) error {
 				}
 
 				var keys []string
-				for k, _ := range r.Labels {
+				for k := range r.Labels {
 					keys = append(keys, k)
 				}
 				sort.Strings(keys)
@@ -949,7 +949,8 @@ func withBatches(templated *state.HelmState, batches [][]state.Release, helm hel
 
 		var releaseIds []string
 		for _, r := range targets {
-			releaseIds = append(releaseIds, state.ReleaseToID(&r))
+			release := r
+			releaseIds = append(releaseIds, state.ReleaseToID(&release))
 		}
 
 		logger.Debugf("processing releases in group %d/%d: %s", i+1, numBatches, strings.Join(releaseIds, ", "))
@@ -959,7 +960,7 @@ func withBatches(templated *state.HelmState, batches [][]state.Release, helm hel
 
 		processed, errs := converge(&batchSt, helm)
 
-		if errs != nil && len(errs) > 0 {
+		if len(errs) > 0 {
 			return false, errs
 		}
 
@@ -998,12 +999,7 @@ func (a *App) visitStatesWithSelectorsAndRemoteSupport(fileOrDir string, converg
 		opts.Environment.OverrideValues = envvals
 	}
 
-	dir, err := a.getwd()
-	if err != nil {
-		return err
-	}
-
-	a.remote = remote.NewRemote(a.Logger, dir, a.readFile, a.directoryExistsAt, a.fileExistsAt)
+	a.remote = remote.NewRemote(a.Logger, "", a.readFile, a.directoryExistsAt, a.fileExistsAt)
 
 	f := converge
 	if opts.Filter {
@@ -1186,7 +1182,8 @@ func (a *App) getSelectedReleases(r *Run, includeTransitiveNeeds bool) ([]state.
 	// We iterate over allReleases rather than groupsByID
 	// to preserve the order of releases
 	for _, seq := range allReleases {
-		id := state.ReleaseToID(&seq)
+		release := seq
+		id := state.ReleaseToID(&release)
 
 		rs := groupsByID[id]
 
@@ -1203,13 +1200,10 @@ func (a *App) getSelectedReleases(r *Run, includeTransitiveNeeds bool) ([]state.
 		// releases in the helmfile config.
 		// Otherwise we can't compute the DAG of releases correctly.
 		r, deduped := selectedIds[id]
-		if !deduped {
-			panic(fmt.Errorf("assertion error: release %q has never been selected. This shouldn't happen!", id))
+		if deduped {
+			deduplicated = append(deduplicated, r)
+			dedupedBefore[id] = struct{}{}
 		}
-
-		deduplicated = append(deduplicated, r)
-
-		dedupedBefore[id] = struct{}{}
 	}
 
 	if err := checkDuplicates(r.helm, r.state, deduplicated); err != nil {
@@ -1291,11 +1285,12 @@ func (a *App) apply(r *Run, c ApplyConfigProvider) (bool, bool, []error) {
 
 	releasesWithNoChange := map[string]state.ReleaseSpec{}
 	for _, r := range toApplyWithNeeds {
-		id := state.ReleaseToID(&r)
+		release := r
+		id := state.ReleaseToID(&release)
 		_, uninstalled := releasesToBeDeleted[id]
 		_, updated := releasesToBeUpdated[id]
 		if !uninstalled && !updated {
-			releasesWithNoChange[id] = r
+			releasesWithNoChange[id] = release
 		}
 	}
 
@@ -1341,7 +1336,8 @@ Do you really want to apply?
 				var rs []state.ReleaseSpec
 
 				for _, r := range subst.Releases {
-					if r2, ok := releasesToBeDeleted[state.ReleaseToID(&r)]; ok {
+					release := r
+					if r2, ok := releasesToBeDeleted[state.ReleaseToID(&release)]; ok {
 						rs = append(rs, r2)
 					}
 				}
@@ -1351,7 +1347,7 @@ Do you really want to apply?
 				return subst.DeleteReleasesForSync(&affectedReleases, helm, c.Concurrency())
 			}))
 
-			if deletionErrs != nil && len(deletionErrs) > 0 {
+			if len(deletionErrs) > 0 {
 				syncErrs = append(syncErrs, deletionErrs...)
 			}
 		}
@@ -1362,7 +1358,8 @@ Do you really want to apply?
 				var rs []state.ReleaseSpec
 
 				for _, r := range subst.Releases {
-					if r2, ok := releasesToBeUpdated[state.ReleaseToID(&r)]; ok {
+					release := r
+					if r2, ok := releasesToBeUpdated[state.ReleaseToID(&release)]; ok {
 						rs = append(rs, r2)
 					}
 				}
@@ -1379,7 +1376,7 @@ Do you really want to apply?
 				return subst.SyncReleases(&affectedReleases, helm, c.Values(), c.Concurrency(), &syncOpts)
 			}))
 
-			if updateErrs != nil && len(updateErrs) > 0 {
+			if len(updateErrs) > 0 {
 				syncErrs = append(syncErrs, updateErrs...)
 			}
 		}
@@ -1410,16 +1407,18 @@ func (a *App) delete(r *Run, purge bool, c DestroyConfigProvider) (bool, []error
 
 	releasesToDelete := map[string]state.ReleaseSpec{}
 	for _, r := range toDelete {
-		id := state.ReleaseToID(&r)
-		releasesToDelete[id] = r
+		release := r
+		id := state.ReleaseToID(&release)
+		releasesToDelete[id] = release
 	}
 
 	releasesWithNoChange := map[string]state.ReleaseSpec{}
 	for _, r := range toSync {
-		id := state.ReleaseToID(&r)
+		release := r
+		id := state.ReleaseToID(&release)
 		_, uninstalled := releasesToDelete[id]
 		if !uninstalled {
-			releasesWithNoChange[id] = r
+			releasesWithNoChange[id] = release
 		}
 	}
 
@@ -1455,7 +1454,7 @@ Do you really want to delete?
 				return subst.DeleteReleases(&affectedReleases, helm, c.Concurrency(), purge)
 			}))
 
-			if deletionErrs != nil && len(deletionErrs) > 0 {
+			if len(deletionErrs) > 0 {
 				errs = append(errs, deletionErrs...)
 			}
 		}
@@ -1579,7 +1578,7 @@ func (a *App) lint(r *Run, c LintConfigProvider) (bool, []error, []error) {
 			return lintErrs
 		}))
 
-		if templateErrs != nil && len(templateErrs) > 0 {
+		if len(templateErrs) > 0 {
 			errs = append(errs, templateErrs...)
 		}
 	}
@@ -1631,7 +1630,7 @@ func (a *App) status(r *Run, c StatusesConfigProvider) (bool, []error) {
 			return subst.ReleaseStatuses(helm, c.Concurrency())
 		}))
 
-		if templateErrs != nil && len(templateErrs) > 0 {
+		if len(templateErrs) > 0 {
 			errs = append(errs, templateErrs...)
 		}
 	}
@@ -1680,36 +1679,39 @@ func (a *App) sync(r *Run, c SyncConfigProvider) (bool, []error) {
 
 	releasesToDelete := map[string]state.ReleaseSpec{}
 	for _, r := range toDelete {
-		id := state.ReleaseToID(&r)
-		releasesToDelete[id] = r
+		release := r
+		id := state.ReleaseToID(&release)
+		releasesToDelete[id] = release
 	}
 
 	var toUpdate []state.ReleaseSpec
 	for _, r := range toSyncWithNeeds {
-		if _, deleted := releasesToDelete[state.ReleaseToID(&r)]; !deleted {
-			if r.Installed == nil || *r.Installed {
-				toUpdate = append(toUpdate, r)
-			} else {
-				// TODO Emit error when the user opted to fail when the needed release is disabled,
-				// instead of silently ignoring it.
-				// See https://github.com/roboll/helmfile/issues/1018
+		release := r
+		if _, deleted := releasesToDelete[state.ReleaseToID(&release)]; !deleted {
+			if release.Installed == nil || *release.Installed {
+				toUpdate = append(toUpdate, release)
 			}
+			// TODO Emit error when the user opted to fail when the needed release is disabled,
+			// instead of silently ignoring it.
+			// See https://github.com/roboll/helmfile/issues/1018
 		}
 	}
 
 	releasesToUpdate := map[string]state.ReleaseSpec{}
 	for _, r := range toUpdate {
-		id := state.ReleaseToID(&r)
-		releasesToUpdate[id] = r
+		release := r
+		id := state.ReleaseToID(&release)
+		releasesToUpdate[id] = release
 	}
 
 	releasesWithNoChange := map[string]state.ReleaseSpec{}
 	for _, r := range toSyncWithNeeds {
-		id := state.ReleaseToID(&r)
+		release := r
+		id := state.ReleaseToID(&release)
 		_, uninstalled := releasesToDelete[id]
 		_, updated := releasesToUpdate[id]
 		if !uninstalled && !updated {
-			releasesWithNoChange[id] = r
+			releasesWithNoChange[id] = release
 		}
 	}
 
@@ -1750,7 +1752,8 @@ func (a *App) sync(r *Run, c SyncConfigProvider) (bool, []error) {
 			var rs []state.ReleaseSpec
 
 			for _, r := range subst.Releases {
-				if r2, ok := releasesToDelete[state.ReleaseToID(&r)]; ok {
+				release := r
+				if r2, ok := releasesToDelete[state.ReleaseToID(&release)]; ok {
 					rs = append(rs, r2)
 				}
 			}
@@ -1760,7 +1763,7 @@ func (a *App) sync(r *Run, c SyncConfigProvider) (bool, []error) {
 			return subst.DeleteReleasesForSync(&affectedReleases, helm, c.Concurrency())
 		}))
 
-		if deletionErrs != nil && len(deletionErrs) > 0 {
+		if len(deletionErrs) > 0 {
 			errs = append(errs, deletionErrs...)
 		}
 	}
@@ -1770,8 +1773,9 @@ func (a *App) sync(r *Run, c SyncConfigProvider) (bool, []error) {
 			var rs []state.ReleaseSpec
 
 			for _, r := range subst.Releases {
-				if _, ok := releasesToDelete[state.ReleaseToID(&r)]; !ok {
-					rs = append(rs, r)
+				release := r
+				if _, ok := releasesToDelete[state.ReleaseToID(&release)]; !ok {
+					rs = append(rs, release)
 				}
 			}
 
@@ -1786,7 +1790,7 @@ func (a *App) sync(r *Run, c SyncConfigProvider) (bool, []error) {
 			return subst.SyncReleases(&affectedReleases, helm, c.Values(), c.Concurrency(), opts)
 		}))
 
-		if syncErrs != nil && len(syncErrs) > 0 {
+		if len(syncErrs) > 0 {
 			errs = append(errs, syncErrs...)
 		}
 	}
@@ -1829,11 +1833,12 @@ func (a *App) template(r *Run, c TemplateConfigProvider) (bool, []error) {
 
 	releasesDisabled := map[string]state.ReleaseSpec{}
 	for _, r := range selectedReleasesWithNeeds {
-		id := state.ReleaseToID(&r)
-		if r.Installed != nil && !*r.Installed {
-			releasesDisabled[id] = r
+		release := r
+		id := state.ReleaseToID(&release)
+		if release.Installed != nil && !*release.Installed {
+			releasesDisabled[id] = release
 		} else {
-			toRender = append(toRender, r)
+			toRender = append(toRender, release)
 		}
 	}
 
@@ -1858,6 +1863,7 @@ func (a *App) template(r *Run, c TemplateConfigProvider) (bool, []error) {
 				IncludeCRDs:       c.IncludeCRDs(),
 				OutputDirTemplate: c.OutputDirTemplate(),
 				SkipCleanup:       c.SkipCleanup(),
+				SkipTests:         c.SkipTests(),
 			}
 			return subst.TemplateReleases(helm, c.OutputDir(), c.Values(), args, c.Concurrency(), c.Validate(), opts)
 		}))
@@ -1912,11 +1918,12 @@ func (a *App) writeValues(r *Run, c WriteValuesConfigProvider) (bool, []error) {
 
 	releasesToWrite := map[string]state.ReleaseSpec{}
 	for _, r := range toRender {
-		id := state.ReleaseToID(&r)
-		if r.Installed != nil && !*r.Installed {
+		release := r
+		id := state.ReleaseToID(&release)
+		if release.Installed != nil && !*release.Installed {
 			continue
 		}
-		releasesToWrite[id] = r
+		releasesToWrite[id] = release
 	}
 
 	var errs []error
@@ -2064,7 +2071,7 @@ type context struct {
 }
 
 func (c context) wrapErrs(errs ...error) error {
-	if errs != nil && len(errs) > 0 {
+	if len(errs) > 0 {
 		for _, err := range errs {
 			switch e := err.(type) {
 			case *state.ReleaseError:
@@ -2077,3 +2084,37 @@ func (c context) wrapErrs(errs ...error) error {
 	}
 	return nil
 }
+
+func (a *App) ShowCacheDir(c ListConfigProvider) error {
+	fmt.Printf("Cache directory: %s\n", remote.CacheDir())
+
+	if !directoryExistsAt(remote.CacheDir()) {
+		return nil
+	}
+	dirs, err := os.ReadDir(remote.CacheDir())
+	if err != nil {
+		return err
+	}
+	for _, e := range dirs {
+		fmt.Printf("- %s\n", e.Name())
+	}
+
+	return nil
+}
+
+func (a *App) CleanCacheDir(c ListConfigProvider) error {
+	if !directoryExistsAt(remote.CacheDir()) {
+		return nil
+	}
+	fmt.Printf("Cleaning up cache directory: %s\n", remote.CacheDir())
+	dirs, err := os.ReadDir(remote.CacheDir())
+	if err != nil {
+		return err
+	}
+	for _, e := range dirs {
+		fmt.Printf("- %s\n", e.Name())
+		os.RemoveAll(filepath.Join(remote.CacheDir(), e.Name()))
+	}
+
+	return nil
+}
